@@ -20,6 +20,7 @@
 
 #ifdef GNOME_LIBS
 #include <libgnomeui/libgnomeui.h>
+#include <libgnome/libgnome.h>
 #endif
 
 #include "libdasher.h"
@@ -1195,6 +1196,9 @@ slider_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer data)
 extern "C" gboolean
 button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
+  GdkEventFocus *focusEvent = (GdkEventFocus *) g_malloc(sizeof(GdkEventFocus));
+  gboolean *returnType;
+
 #ifdef WITH_GPE
   // GPE version requires the button to be held down rather than clicked
   if ((event->type != GDK_BUTTON_PRESS) && (event->type != GDK_BUTTON_RELEASE))
@@ -1204,7 +1208,13 @@ button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer data)
     return FALSE;
 #endif
 
+  focusEvent->type = GDK_FOCUS_CHANGE;
+  focusEvent->window = (GdkWindow *) the_canvas;
+  focusEvent->send_event = FALSE;
+  focusEvent->in = TRUE;
 
+  gtk_widget_grab_focus(GTK_WIDGET(the_canvas));
+  g_signal_emit_by_name(GTK_OBJECT(the_canvas), "focus_in_event", GTK_WIDGET(the_canvas), focusEvent, NULL, &returnType);
 
   // CJB,  2003-08.  If we have a selection, replace it with the new input.
   // This code is duplicated in key_press_event.
@@ -1392,7 +1402,7 @@ open_window(GladeXML *xml) {
   dasher_redraw();
 
   // Aim for 20 frames per second
-  g_timeout_add(50, timer_callback, NULL );  
+  g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE,50, timer_callback, NULL,NULL); 
 
   // I have no idea why we need to do this when Glade has theoretically done
   // so already, but...
@@ -1631,6 +1641,7 @@ extern "C" void about_dasher(GtkWidget *widget, gpointer user_data)
     "Chris Ball",
     "Phil Cowens",
     "Matthew Garrett",
+    "David MacKay",
     "Iain Murray",
     "Hanna Wallach",
     "David Ward",
@@ -1639,7 +1650,9 @@ extern "C" void about_dasher(GtkWidget *widget, gpointer user_data)
     
   // Yeah, should really do some Gnome documentation for it...
   gchar *documenters[] = {
+    "Chris Ball",
     "Matthew Garrett",
+    "David MacKay",
     NULL
   };
 
